@@ -93,6 +93,7 @@ public:
    * @return the status code of the write operation
    */
   typedef sigc::slot<ErrorStatus, const unsigned char* /*data*/, unsigned int /*length*/> SlotWriteFunc;
+
   /**
    * This is the type of function which is called when a backend needs to read
    * data from an input stream. It is passed the buffer to read the data into
@@ -115,6 +116,9 @@ public:
    */
   explicit Surface(cairo_surface_t* cobject, bool has_reference = false);
 
+  Surface(const Surface&) = delete;
+  Surface& operator=(const Surface&) = delete;
+
   virtual ~Surface();
 
   /**
@@ -410,10 +414,6 @@ protected:
   /** The underlying C cairo surface type that is wrapped by this Surface
    */
   cobject* m_cobject;
-
-private:
-  Surface(const Surface&);
-  Surface& operator=(const Surface&);
 };
 
 /** @example image-surface.cc
@@ -451,7 +451,7 @@ public:
    */
   explicit ImageSurface(cairo_surface_t* cobject, bool has_reference = false);
 
-  virtual ~ImageSurface();
+  ~ImageSurface() override;
 
   /** Gets the width of the ImageSurface in pixels
    */
@@ -584,6 +584,107 @@ public:
 };
 
 
+/**
+ * A recording surface is a surface that records all drawing operations at the
+ * highest level of the surface backend interface, (that is, the level of paint,
+ * mask, stroke, fill, and show_text_glyphs). The recording surface can then be
+ * "replayed" against any target surface by using it as a source surface.
+ *
+ * If you want to replay a surface so that the results in `target` will be
+ * identical to the results that would have been obtained if the original
+ * operations applied to the recording surface had instead been applied to the
+ * target surface, you can use code like this:
+ *
+ *     Cairo::RefPtr<Cairo::Context> context = Cairo::Context::create(target);
+ *     context->set_source(recording_surface, 0.0, 0.0);
+ *     context->paint();
+ *
+ * A recording surface is logically unbounded, i.e. it has no implicit
+ * constraint on the size of the drawing surface. However, in practice this is
+ * rarely useful as you wish to replay against a particular target surface with
+ * known bounds. For this case, it is more efficient to specify the target
+ * extents to the recording surface upon creation.
+ *
+ * The recording phase of the recording surface is careful to snapshot all
+ * necessary objects (paths, patterns, etc.), in order to achieve accurate
+ * replay.
+ *
+ * Note that like all surfaces, a RecordingSurface is a reference-counted object
+ * that should be used via Cairo::RefPtr.
+ */
+class RecordingSurface : public Surface
+{
+public:
+
+  /**
+   * Create a C++ wrapper for the C instance. This C++ instance should then be
+   * given to a RefPtr.
+   * @param cobject The C instance.
+   * @param has_reference Whether we already have a reference. Otherwise, the
+   * constructor will take an extra reference.
+   */
+  explicit RecordingSurface(cairo_surface_t* cobject, bool has_reference = false);
+
+  virtual ~RecordingSurface();
+
+  /**
+   * Measures the extents of the operations stored within the recording
+   * surface.  This is useful to compute the required size of an image surface
+   * (or equivalent) into which to replay the full sequence of drawing
+   * operations.
+   * @return a Rectangle with `x` and `y` set to the coordinates of the top-left
+   * of the ink bounding box, and `width` and `height` set to the width and
+   * height of the ink bounding box.
+   */
+  Rectangle ink_extents() const;
+
+  /**
+   * Get the extents of the recording surface, if the surface is bounded.
+   * @param extents the Rectangle in which to store the extents if the recording
+   * surface is bounded
+   * @return true if the recording surface is bounded, false if the recording
+   * surface is unbounded (in which case `extents` will not be set).
+   */
+  bool get_extents(Rectangle &extents) const;
+
+  /**
+   * Creates a recording surface which can be used to record all drawing
+   * operations at the highest level (that is, the level of paint, mask, stroke,
+   * fill and show_text_glyphs). The recording surface can then be "replayed"
+   * against any target surface by using it as a source to drawing operations.
+   * The recording surface will be unbounded.
+   *
+   * The recording phase of the recording surface is careful to snapshot all
+   * necessary objects (paths, patterns, etc.), in order to achieve accurate
+   * replay.
+   *
+   * @param content the content of the recording surface; defaults to
+   * Cairo::Content::CONTENT_COLOR_ALPHA.
+   * @return a RefPtr to the newly created recording surface.
+   */
+  static RefPtr<RecordingSurface> create(Content content = Content::CONTENT_COLOR_ALPHA);
+
+  /**
+   * Creates a recording surface which can be used to record all drawing
+   * operations at the highest level (that is, the level of paint, mask, stroke,
+   * fill and show_text_glyphs). The recording surface can then be "replayed"
+   * against any target surface by using it as a source to drawing operations.
+   * The recording surface will be bounded by the given rectangle.
+   *
+   * The recording phase of the recording surface is careful to snapshot all
+   * necessary objects (paths, patterns, etc.), in order to achieve accurate
+   * replay.
+   *
+   * @param	extents the extents to record
+   * @param	content the content of the recording surface; defaults to
+   * Cairo::Content::CONTENT_COLOR_ALPHA.
+   * @return	a RefPtr to the newly created recording surface.
+   */
+  static RefPtr<RecordingSurface> create(const Rectangle& extents, Content content = Content::CONTENT_COLOR_ALPHA);
+
+};
+
+
 #ifdef CAIRO_HAS_PDF_SURFACE
 
 /** @example pdf-surface.cc
@@ -615,7 +716,7 @@ public:
    * constructor will take an extra reference.
    */
   explicit PdfSurface(cairo_surface_t* cobject, bool has_reference = false);
-  virtual ~PdfSurface();
+  ~PdfSurface() override;
 
   /** Creates a PdfSurface with a specified dimensions that will be saved as
    * the given filename
@@ -718,7 +819,7 @@ public:
    * constructor will take an extra reference.
    */
   explicit PsSurface(cairo_surface_t* cobject, bool has_reference = false);
-  virtual ~PsSurface();
+  ~PsSurface() override;
 
   /** Creates a PsSurface with a specified dimensions that will be saved as the
    * given filename
@@ -873,7 +974,7 @@ public:
    * constructor will take an extra reference.
    */
   explicit SvgSurface(cairo_surface_t* cobject, bool has_reference = false);
-  virtual ~SvgSurface();
+  ~SvgSurface() override;
 
 
   /** Creates a SvgSurface with a specified dimensions that will be saved as the
@@ -962,7 +1063,7 @@ public:
    */
   explicit GlitzSurface(cairo_surface_t* cobject, bool has_reference = false);
 
-  virtual ~GlitzSurface();
+  ~GlitzSurface() override;
 
   /** Creates a new GlitzSurface
    *
